@@ -1,4 +1,4 @@
-# Copyright (c) 2019-2021, Jonas Eschle, Jim Pivarski, Eduardo Rodrigues, and Henry Schreiner.
+# Copyright (c) 2019-2024, Jonas Eschle, Jim Pivarski, Eduardo Rodrigues, and Henry Schreiner.
 #
 # Distributed under the 3-clause BSD license, see accompanying file LICENSE
 # or https://github.com/scikit-hep/vector for details.
@@ -8,6 +8,7 @@
 
     Planar.rotateZ(self, angle)
 """
+
 from __future__ import annotations
 
 import typing
@@ -47,5 +48,8 @@ def dispatch(angle: typing.Any, v: typing.Any) -> typing.Any:
     function, *returns = _from_signature(__name__, dispatch_map, (_aztype(v),))
     with numpy.errstate(all="ignore"):
         return v._wrap_result(
-            _flavor_of(v), function(v.lib, angle, *v.azimuthal.elements), returns, 1
+            _flavor_of(v),
+            function(v.lib, angle, *v.azimuthal.elements),
+            returns,
+            1,
         )
